"""
:author: Gatsby Lee
:since: 2019-05-30

Interface for retry functions
"""

from tenacity import (
    retry,
    retry_all,
    retry_always,
    retry_any,
    retry_if_exception,
    retry_if_exception_message,
    retry_if_exception_type,
    retry_if_not_exception_message,
    retry_if_not_result,
    retry_if_result,
    retry_never,
    retry_unless_exception_type,
    stop,
    stop_after_attempt,
    stop_after_delay,
    stop_all,
    stop_any,
    stop_never,
    stop_when_event_set,
    wait,
    wait_chain,
    wait_exponential,
    wait_fixed,
    wait_full_jitter,
    wait_incrementing,
    wait_none,
    wait_random,
    wait_random_exponential,
    after_log,
    before_log,
    before_sleep_log,
)

__all__ = (
    'retry',
    'retry_all',
    'retry_always',
    'retry_any',
    'retry_if_exception',
    'retry_if_exception_message',
    'retry_if_exception_type',
    'retry_if_not_exception_message',
    'retry_if_not_result',
    'retry_if_result',
    'retry_never',
    'retry_unless_exception_type',
    'stop',
    'stop_after_attempt',
    'stop_after_delay',
    'stop_all',
    'stop_any',
    'stop_never',
    'stop_when_event_set',
    'wait',
    'wait_chain',
    'wait_exponential',
    'wait_fixed',
    'wait_full_jitter',
    'wait_incrementing',
    'wait_none',
    'wait_random',
    'wait_random_exponential',
    # logging
    'after_log',
    'before_log',
    'before_sleep_log',
)



"""
:author: Gatsby Lee
:since: 2019-05-29
"""
from retry_redis.__about__ import __version__
from retry_redis.decorated_redis import (
    Redis,
)


def int_or_str(value):
    try:
        return int(value)
    except ValueError:
        return value


VERSION = tuple(map(int_or_str, __version__.split('.')))


__all__ = (
    '__version__',
    'VERSION',
    'Redis',
)



"""
:author: Gatsby Lee
:since: 2019-08-12
"""
from tenacity import _utils
from tenacity import compat as _compat
from tenacity.wait import wait_base


class wait_exponential_onlyif_exception_type(wait_base):

    def __init__(self, wait_onlyif_exception=Exception,
                 multiplier=1, max=_utils.MAX_WAIT, exp_base=2, min=0):
        self.exception_types = wait_onlyif_exception
        self.multiplier = multiplier
        self.min = min
        self.max = max
        self.exp_base = exp_base

    @_compat.wait_dunder_call_accept_old_params
    def __call__(self, retry_state):

        e = retry_state.outcome.exception()
        if not isinstance(e, self.exception_types):
            return 0

        try:
            exp = self.exp_base ** (retry_state.attempt_number - 1)
            result = self.multiplier * exp
        except OverflowError:
            return self.max
        return max(max(0, self.min), min(result, self.max))



"""
:author: Gatsby Lee
:since: 2019-05-29
"""
import logging

import redis

from retry_redis.retry_decorator import (
    before_sleep_log,
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
)

LOGGER = logging.getLogger(__name__)

RETRY_EXCEPTIONS = (
    redis.exceptions.ConnectionError,
    redis.exceptions.ResponseError,
    redis.exceptions.TimeoutError,
)

RETRY_DECORATOR = retry(
    retry=retry_if_exception_type(RETRY_EXCEPTIONS),
    wait=wait_exponential(multiplier=1),
    stop=stop_after_attempt(3),
    before_sleep=before_sleep_log(LOGGER, logging.DEBUG),
    reraise=True
)


class Redis(object):

    def __init__(self, **kwargs):
        """
        In order to match keyword arguments to redis-py one,
        custom arguments must not be added.
        """
        self._redis = redis.Redis(**kwargs)
        self._retry_decorator = RETRY_DECORATOR

    def __repr__(self):
        return "%s<%s>" % (type(self).__name__, repr(self._redis))

    def __getattr__(self, key):
        """
        Override to bind function call to redis-py
        """
        redis_method = getattr(self._redis, key)
        return self._retry_decorator(redis_method)


__all__ = (
    'Redis',
)



from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

version = {}
with open(path.join(here, 'retry_redis/__about__.py'), encoding='utf-8') as f:
    exec(f.read(), version)

requires = [
    'redis',
    'tenacity',
]

test_requires = [
    'pytest',
]

dev_requires = test_requires + [
    'wheel',
    'bpython',
    'pytest',
    'flake8',
    'autopep8',
    'pylint'
]

setup(
    name='retry-redis',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version=version['__version__'],

    description='Redis Clieht with customizable retrying capability.',
    long_description=long_description,
    long_description_content_type='text/x-rst',

    # The project's main homepage.
    url='https://github.com/Gatsby-Lee/retry-redis',

    # Author details
    author='Gatsby Lee',

    # Choose your license
    license='GPL-3.0',

    # See https://pypi.org/pypi?%3Aaction=list_classifiers
    classifiers=[

        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Utilities',
    ],
    # What does your project relate to?
    keywords='redis retry failure',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/technical.html#install-requires-vs-requirements-files
    python_requires=">=3.6",
    install_requires=requires,
    extras_require={
        'dev': dev_requires,
        'test': test_requires,
    },
)
